package com.qmth.distributed.print.business.templete.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.distributed.print.business.service.DataSyncService;
import com.qmth.distributed.print.business.service.impl.DataSyncServiceImpl;
import com.qmth.distributed.print.business.templete.service.PushLogicService;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.params.PushBeforeRoleParam;
import com.qmth.teachcloud.common.bean.params.UserPushParam;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.userPush.BeforeJudgeEnum;
import com.qmth.teachcloud.common.enums.userPush.SyncStatusEnum;
import com.qmth.teachcloud.common.service.SysUserRoleService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.sync.StmmsUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: 同步推送处理逻辑接口实现类
 * @Author: CaoZixuan
 * @Date: 2021-11-01
 */
@Service
public class PushLogicServiceImpl implements PushLogicService {
    private final static Logger log = LoggerFactory.getLogger(PushLogicServiceImpl.class);

    @Resource
    private SysUserService sysUserService;

    @Resource
    private SysUserRoleService sysUserRoleService;

    @Resource
    StmmsUtils stmmsUtils;

    @Override
    public Map<String, Object> executeUserPushLogic(Map<String, Object> map) throws IllegalAccessException {
        int correct = 0;
        int count = 0;
        SysUser sysUser = (SysUser) map.get(SystemConstant.SYS_USER);
        Long schoolId = sysUser.getSchoolId();
        // 需要同步的用户
        List<SysUser> sysUserList = sysUserService.list(new QueryWrapper<SysUser>().lambda()
                .eq(SysUser::getSchoolId, schoolId)
                .eq(SysUser::getSyncStatus, SyncStatusEnum.NEED_PUSH_AGAIN));
        count = sysUserList.size();
        for (SysUser user : sysUserList) {
            Set<Long> roleIdSet = sysUserRoleService.listRoleByUserId(user.getId()).stream().map(BaseEntity::getId).collect(Collectors.toSet());
            PushBeforeRoleParam pushBeforeRoleParam = new PushBeforeRoleParam(null, BeforeJudgeEnum.FORBIDDEN_NECESSARY);
            List<UserPushParam> userPushParamList = sysUserService.analyzeUserPushSpecialPrivilege(user.getId(), user.getSchoolId(), user.getCode(), user.getRealName(), user.getPassword(), roleIdSet, pushBeforeRoleParam, user.getEnable());
            boolean syncResult = sysUserService.userPushService(userPushParamList, sysUser);
            if (syncResult) {
                correct++;
            }
            UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(SysUser::getSyncStatus, null).eq(SysUser::getId, user.getId());
            sysUserService.update(updateWrapper);
        }
        map.put("correct", correct);
        map.put("count", count);
        return map;
    }

    /**
     * 成绩查询推送异步任务处理方法
     *
     * @param map 任务源
     * @return
     * @throws IllegalAccessException
     */
    @Override
    public Map<String, Object> executeScorePushLogic(Map<String, Object> map) throws Exception {
        Integer examId = (Integer) map.get("examId");
        String examCode = (String) map.get("examCode");
        try {
            int totalCount = stmmsUtils.getStudentCount(examId, examCode, null, null, null, null, null, null);
            log.info("云阅卷：考试成绩考生数量查询接口调用，返回数量：{}", totalCount);
//            List<Map> paramList = new ArrayList<>();
            if (totalCount > 0) {
                int pageSize = stmmsUtils.getDefaultPageSize();
                int mod = totalCount % pageSize;
                int pageNos = mod == 0 ? totalCount / pageSize : totalCount / pageSize + 1;
                for (int i = 1; i <= pageNos; i++) {
                    List<Map> students = stmmsUtils.getStudentScore(examId, examCode, null, null, null, null, null, null, i, pageSize);
                    for (Map student : students) {
                        String jxbId = String.valueOf(student.get("className"));
                        String xhId = String.valueOf(student.get("studentCode"));
                        String xmcj = String.valueOf(student.get("totalScore"));
//                        Map<String, String> stringMap = new HashMap<>();
//                        stringMap.put("jxb_id", jxbId);
//                        stringMap.put("xh_id", xhId);
//                        stringMap.put("xmcj", xmcj);
//                        paramList.add(stringMap);
                    }
                }
            }
            map.computeIfAbsent("count", v -> totalCount);
            map.computeIfAbsent("correct", v -> totalCount);
        } catch (Exception e) {
            log.info("云阅卷：接口调用失败：{}", e.getMessage());
            throw new RuntimeException("云阅卷：接口调用失败：" + e.getMessage());
        }
        return map;
    }
}
